import ast
import numpy as np
from sentence_transformers import SentenceTransformer
from k_means_constrained import KMeansConstrained
from sklearn.cluster import KMeans

class Clustering():

  def __init__(self, all_key_words, model='all-mpnet-base-v2', salto=8):
    # Parametros necesarios para el clustering
    self.wss = []
    self.kmeans_labels = []
    self.x_label = []
    self.corpus_embeddings = []
    self.salto = salto
    self.embedder = SentenceTransformer(model)
    self.all_key_words = all_key_words

    # Creamos el corpus embeddign (representaciones vectoriales de las keys extraidas)
    self.__sentence_embedding_model()

  def __sentence_embedding_model(self):
    # Generamos el embeddings para cada una de las keys o ngramas.
    self.corpus_embeddings = self.embedder.encode(self.all_key_words)

    # Normaliza las incrustaciones para que tengan una longitud unitaria
    self.corpus_embeddings = self.corpus_embeddings / np.linalg.norm(self.corpus_embeddings, axis=1, keepdims=True)

  def kmeans_clustering(self):
    # Por cada numero de clusters posible aplicamos Kmeans al embedding en cuestion
    for i in range(2, self.corpus_embeddings.shape[0], self.salto):
      k_mean = KMeans(n_clusters=i)
      k_mean.fit(self.corpus_embeddings)
      self.wss.append(k_mean.inertia_)
      self.kmeans_labels.append(k_mean.labels_)
      self.x_label.append(i)

    inertia_diff = np.diff(self.wss)
    normalized_diff = inertia_diff / self.wss[:-1]
    elbow_strength = np.max(normalized_diff)
    # Tenemos el indice que es mayor
    optimal_cluster = np.argmax(normalized_diff)  # Add 2 because of the np.diff

    # El indice representa el numero de saltos de 14 posicones que hemos hehco
    # Por eso multiplicamos * 2 y le sumamos 2 ya que empieza en ese valor
    opt_cls = optimal_cluster * self.salto + 2

    # Una vez tenemos el numero de clusters optimo recuperamos las labels del kMeans.
    # y obtenemos los clusters
    etiquetas = self.kmeans_labels[optimal_cluster]
    clustered_sentences = [[] for i in range(opt_cls)]
    all_key_words = list(self.all_key_words)

    for sentence_id, cluster_id in enumerate(etiquetas):
      clustered_sentences[cluster_id].append(all_key_words[sentence_id])

    return clustered_sentences

  def kmeans_constrained_clustering(self, size_min = 4):

    #utilizamos el método de Elbow para encontrar el numero optimo de cluster teniendo en cuenta que queremos
    # que como minimo tengan un tamaño de cuatro puntos cada uno de los cluters.
    for i in range(2, (self.corpus_embeddings.shape[0]//size_min), self.salto):
      clf = KMeansConstrained(
        n_clusters=i, # Num de clusters que queremos
        size_min=size_min, # Min num de puntos que tendra el cluster
        random_state=42 # Semilla de generacion necesaria para repetir un clusterizado dando el mismo resultado
      )
      # Le pasamos los datos que queremos hacer el clusterizado
      clf.fit(self.corpus_embeddings)
      self.wss.append(clf.inertia_) # Guardamos la suma al cuadrado de las distancias de los puntos con su centroide
      self.kmeans_labels.append(clf.labels_) #Guardamos el cluster al que le toca cada punto
      self.x_label.append(i) # Guardamos el num de clusters en cuestion

    inertia_diff = np.diff(self.wss)
    normalized_diff = inertia_diff / self.wss[:-1]
    # Tenemos el indice que es mayor
    optimal_cluster = np.argmax(normalized_diff)

    # El indice representa el numero de saltos de x posicones que hemos hecho
    # Por eso multiplicamos * salto y le sumamos 2 ya que empieza en ese valor
    opt_clf = optimal_cluster * self.salto + 2

    # Una vez tenemos el numero de clusters optimo recuperamos las labels del kMeans.
    # y obtenemos los clusters
    etiquetas = self.kmeans_labels[optimal_cluster]
    clustered_sentences = [[] for i in range(opt_clf)]
    all_key_words = list(self.all_key_words)

    for sentence_id, cluster_id in enumerate(etiquetas):
      clustered_sentences[cluster_id].append(all_key_words[sentence_id])

    return clustered_sentences

  def load_cluster(self, path):
    # En este método cargaremos una clusterizacion ya creada a partir de un archivo txt.
    with open(path, 'r', encoding='utf-8') as file:
      cls = file.readlines()

    # Construimos el string
    lst_cluster = '['
    for line in cls:
      lst_cluster += str(line[line.index('['):]) + ','

    # Cerramos la lista
    lst_cluster += ']'
    #Apartir del str que hemos cargado creamos la lista de listas
    lst_cluster = ast.literal_eval(lst_cluster)
    return lst_cluster


