import os
import random
from dotenv import load_dotenv
from openai import OpenAI

class Definitions():
    def __init__(self):
        # Obtener la clave de API desde las variables de entorno
        load_dotenv()
        api_key = os.getenv("API_KEY")
        # Configurar la clave de API para OpenAI
        self.openai = OpenAI(api_key=api_key)

    def get_definition(self, key):

        # Iteramos sobre cada una de las llaves
        # Creamos el prompt para preguntar a GPT-3.5
        prompt = "Provide a definition for the following concept." +\
        "The definition must not contain the name of the concept.\n\nConcept: {}".format(key)


        # Generamos la consulta para OpneAI
        response = self.openai.chat.completions.create(
            model="gpt-3.5-turbo",
            messages=[
                {"role": "system", "content": "You are a helpful assistant."},
                {"role": "user",
                 "content": prompt}
            ],
            max_tokens=100,
            stream=True,
        )

        # Guardamos la respuesta que nos devuelve la API
        text = ""
        for chunk in response:
            if chunk.choices[0].delta.content is not None:
                text += chunk.choices[0].delta.content

        return text

    def randomize_cuestion(self, lst_keys, definitions):
        # String donde se guardara el string con la pregunta y sus respuestas
        question = ""

        # Queremos generar por cada una de las keys presentes en los clusters una pregunta multi respuesta

        for num in range(len(lst_keys)):
            # Seleccionamos aleatoriamente una key de la cual haremos la pregunta
            #num = random.randint(0, len(lst_keys) - 1)

            # Creamos una lista que representara el orden en que apareceran las respuestas a la pregunta
            lst_answers = [i for i in range(len(lst_keys))]

            # Para evitar que la definicion de la key que se hace la pregunta salga mas de una vez eliminamos el número
            # de la lista lst_answers
            lst_answers.remove(num)

            # Una vez eliminada seleccionamos de manera aleatoria otras 3 definiciones
            answers_selected = random.sample(lst_answers, 3)

            # Añadimos la respuesta correcta
            answers_selected.append(num)

            # Ahora mezclamos el orden de las respuestas
            random.shuffle(answers_selected)

            # Finalmente generamos la pregunta
            question += "QUESTION: What is the definition of " + lst_keys[num] + "?\n"

            # Iteramos sobre las definiciones seleccionadas  para la pregunta multi respuesta para ser almacenada en la
            # varaible tipo string
            for i, idx in enumerate(answers_selected):
                # Extraemos la definicion
                definition = definitions[idx]

                # De la definicion extraemos el nombre del concepto y lo substituimos por el de la pregunta
                if idx != num:
                    # Para añadir dificultad a la pregunta en caso de que GPT haya añadido el nombre del conceptop en su
                    # definicion substituiremos ese nombre por el nombre del concepto por le cual se esta haciendo la
                    # pregunta, de esta manera consegguimos un aumento de dificultad de la pregunta .
                    definition = definition.replace(lst_keys[idx], lst_keys[num])

                    # Pude darse la casialidad de que el nombre del concepto este capitalizado entonces el replace anterior
                    # puede no funcionar, en ese caso capitalizaremos la palabra
                    definition = definition.replace(lst_keys[idx].capitalize(), lst_keys[num].capitalize())
                else:
                    # En caso de que el concepto sea el cual se esta haciendo la pregunta añadimos un correct para
                    # marcar que es la respuesta correcta
                    definition += " (Correct answer)"

                question += str(i+1) + ': ' + definition + '\n'

            # Añadimos dos saltos de linea entre pregunta y pregunta para que se diferencien claramente en el documento
            # final donde se guardara
            question += '\n\n'

        return question