from langchain.text_splitter import RecursiveCharacterTextSplitter
from keybert import KeyBERT

class KeyphraseExtractionPipeline():
    def __init__(self,chunk_size = 3500, model = 'all-mpnet-base-v2'):
      self.kw_model = KeyBERT(model=model)
      self.text_splitter = RecursiveCharacterTextSplitter(
        chunk_size=chunk_size,
        chunk_overlap=20,
        length_function=len,
        is_separator_regex=False,
      )

    def extract_keys(self,path):
      with open(path, 'r', encoding='utf-8') as fp:
        doc = fp.read()

      # Partimos el texto
      text_chunks = self.text_splitter.create_documents([doc])

      # Set que contendra los ngramas encontrados
      all_key_words = set()

      # Por cada chunk del texto obtenemos las Keys o Ngramas que describen a este.
      for chunk in text_chunks:
        keywords = self.kw_model.extract_keywords(
          chunk.page_content, keyphrase_ngram_range=(1, 2),
          top_n=3, stop_words='english'
        )

        for word in keywords:
          if word[0] not in all_key_words:
            all_key_words.add(word[0])

      return list(all_key_words)

