import re
import os
import string
from sentence2sentence.fine_tunne import FineTuningSeq2Seq
from transformers import GPT2LMHeadModel,GPT2Tokenizer
from keyphrase.keyphrase_extractor import KeyphraseExtractionPipeline
from clustering.clustering import Clustering
from pdf_to_txt.pdf_to_txt import pdf_to_text
from definitions_generator.openAI import Definitions
from random import randint

def train_model_sentence2sentence_v2():
    path = "data/train.txt"
    sen2sen = FineTuningSeq2Seq(path,"gpt2")
    sen2sen.load_dataset()
    sen2sen.train()
    
def keyphrase_exctraction():
      extractor = KeyphraseExtractionPipeline()
      script_dir = os.path.dirname(__file__)
      path = os.path.join(script_dir, "data\\train_for_keyextraction.txt")
      keyphrases = extractor.extract_keys(path)
      return keyphrases

def get_clustering(all_key_words, type = 1, size_min=4):
    if type==1:
        clustering = Clustering(all_key_words)
        clusters = clustering.kmeans_clustering()
    elif type==2:
        clustering = Clustering(all_key_words)
        clusters = clustering.kmeans_constrained_clustering(size_min)
    else:
        clustering = Clustering(all_key_words)
        clusters = clustering.load_cluster('./data/Cluster_result.txt')
    return clusters

def get_output(keys,model_path = 'newModel\\model_tfg_nou'):
    tokenizer = GPT2Tokenizer.from_pretrained('gpt2')
    model = GPT2LMHeadModel.from_pretrained(model_path)
    result = "" 
    answer_question = list()
    question_list = list()
    for i in keys:
        sequence = "What is the definition of "+ i + "?"
        question_list.append(sequence)
        print(sequence)
        ids = tokenizer.encode(f'{sequence}', return_tensors='pt')
        final_outputs = model.generate(
            ids,
            do_sample=True,
            max_length=100,  
            pad_token_id=model.config.eos_token_id,
            top_k=50,
            top_p=0.95,
        )
        final_str = ""
        answer = tokenizer.decode(final_outputs[0], skip_special_tokens=True)
        for char in answer:
            if char in string.printable:
                final_str += char
            final_str = final_str.replace("\n", " ")
            final_str = re.sub(r"\s", " ", final_str)
        final_str.strip()
        result += "QUESTION: "+ final_str +"\n"
        answer_question.append(final_str.replace(sequence,""))
    result += "MULTICHOICE QUESTION\n"
    value = randint(0, (len(keys)-1))
    result += question_list[value] + "\n"
    for i in answer_question:
        result += i + "\n"
    print(result)
    result += "TRUE/FALSE QUESTION\n"
    value = randint(0, (len(keys)-1))
    result += "The definition of " + keys[value] + " is...\n"
    value = randint(0, (len(keys)-1))
    result += answer_question[value]
    
    print(result)
    with open('results\\answers.txt', 'w') as f:
        f.write(result)

def get_api_definitions(keys):

    # Instanciamos la clase
    model = Definitions()

    # Declaramos la lista de cuestiones y respuestas
    answer_question = list()
    question_list = list()
    result =""

    # Iteramos sobre cada una de las que queremos obtener la definicion de la API
    for key in keys:
        sequence = "What is the definition of "+ key + "?"
        question_list.append(sequence)
        response = model.get_definition(key)

        # Guardamos la informacion
        question_list.append(sequence)
        answer_question.append(response)
        result +=  sequence + "\n" + response + "\n \n"

    result += model.randomize_cuestion(keys, answer_question)

    # Generamos la pregunta multi respuesta
    with open('results\\answers.txt', 'w') as f:
        f.write(result)


def get_pdf_to_txt_converter():
    converter = pdf_to_text()
    converter.pdf_txt()

def menu():
    while(True):
        print("1. Transform pdf to txt")
        print("2. Fine tunning")
        print("3. Keyphrase Extraction")
        print("4. Embedding and clustering")
        print("5. Get definitions")

        param = True
        while(param):
            try:
                answer = int(input())
                param = False
            except ValueError:
                # Handle the exception
                print('Please enter an integer')
                param = True
        if (answer == 1):
            get_pdf_to_txt_converter()
            print("PDF convertido con exito!!!")
        if(answer == 2):
            print("The file in txt have to be in the project_dir/data.CONTINUE?(Y/N)")
            answer = input()
            if(answer == "Y"):
                print("TRAINING GPT2 MODEL")
                train_model_sentence2sentence_v2()
        if(answer == 3):
            print("Starting the Key Extraction this will take a few minutes...")
            keys = keyphrase_exctraction()
            with open('Key_extraction.txt', 'w', encoding='utf-8' ) as fp:
                fp.write(str(keys))
            delation = True
            while(delation):
                print("Las llaves",keys)
                print("Total de",len(keys),"extraidas")
                param = True
                while(param):
                    print("Give me the id of the key you want to delete or press -1")
                    try:
                        answer = int(input())
                        param = False
                    except ValueError:
                        # Handle the exception
                        print('Please enter an integer')
                        param = True
                    if (answer == -1):
                        delation = False
                    else:
                        try:
                            keys.pop(answer)
                        except IndexError:
                            print("Out of range ")
        if(answer == 4):
            try:
                if(keys):
                    print("What type of Clustering you want to use")
                    print("1. KMeans")
                    print("2. Constrained KMeans")
                    print("3. Load Cluster from file")
                    type_cls = 0
                    while not (type_cls>0 and type_cls<=3):
                        try:
                            type_cls = int(input("Type:"))
                        except ValueError:
                            # Handle the exception
                            print('Please enter an integer')
                    # Minimo tamaño de los clusters
                    min_size = 4
                    if(type_cls == 2):
                        try:
                            print("Specify the minimum size of clusters for the Constrained KMeans")
                            min_size = int(input("min size cluster:"))
                        except ValueError:
                            print('Error ocurred the min size will be 4')

                    print("Initiating the clusterization this will take a few minutes...")
                    clustering = get_clustering(keys, type_cls, min_size)

                    # Guardamos todos los clusters dentro de un archivo txt
                    for i, cluster in enumerate(clustering):
                        print("Cluster ", (i+1), cluster)
                    with open('./data/Cluster_result.txt', 'w', encoding='utf-8') as f:
                        for i, cluster in enumerate(clustering):
                            # Escribir el número de cluster
                            f.write("Cluster " + str(i) + ": ")
                            # Escribir los elementos del cluster separados por comas
                            f.write(str(cluster))
                            f.write("\n")  # Agregar un salto de línea después de cada cluster
                else:
                    print("Please first get the keyphrase!!!")
            except UnboundLocalError:
                # Maneja el error aquí o simplemente omite la excepción
                print("Get the keyphrase")
                pass
        if(answer == 5):
            print("What group get definitions")
            param = True
            while param:
                try:
                    answer = int(input())
                    answer -= 1
                    if 0 <= answer < len(clustering):
                        param = False
                    else:
                        print('Index out of range. Please enter a valid index.')
                        param = True
                except ValueError:
                    # Handle the exception
                    print('Please enter an integer')
                    param = True

            # Seleccionamos que modelo queremos usar para generar las definiciones
            print("What model of LLM want to use")
            print("1. GPT-2 (Fine-Tuning) ")
            print("2. GPT-3.5-turbo (Need API key)")
            param = True
            option = 0
            while param:
                try:
                    option = int(input())
                    if 0 < option < 3 :
                        param = False
                    else:
                        print('Please select a valid option')
                        param = True
                except ValueError:
                    # Handle the exception
                    print('Please enter an integer')
                    param = True

            if option == 1:
                get_output(clustering[answer],'gpt2')

            elif option == 2:
                get_api_definitions(clustering[answer])




if __name__ == '__main__':
    menu()

    #vector_words = keyphrase_exctraction()
    #vector_embedding = get_embedding(vector_words)
    #clustering = get_clustering(vector_embedding,vector_words)

     
    