from transformers import GPT2LMHeadModel, GPT2Tokenizer
from transformers import Trainer, TrainingArguments
from transformers import TextDataset, DataCollatorForLanguageModeling
import torch
import os


class FineTuningSeq2Seq:
    def __init__(self, path, model_name):
        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
        self.train_file_path = path
        self.output_dir = os.path.join(os.path.dirname(__file__), "..\\newModel")
        self.tokenizer = GPT2Tokenizer.from_pretrained(model_name)
        self.model = GPT2LMHeadModel.from_pretrained(model_name).to(self.device)

    def load_dataset(self):
        self.dataset = TextDataset(
            tokenizer=self.tokenizer,
            file_path=self.train_file_path,
            block_size=128,
        )

    def train(self):
        data_collator = DataCollatorForLanguageModeling(tokenizer=self.tokenizer, mlm=False)
        self.tokenizer.save_pretrained(self.output_dir)
        self.model.save_pretrained(self.output_dir)

        training_args = TrainingArguments(
            output_dir=self.output_dir,
            overwrite_output_dir=False,
            per_device_train_batch_size=8,
            num_train_epochs=50.0,
        )

        trainer = Trainer(
            model=self.model,
            args=training_args,
            data_collator=data_collator,
            train_dataset=self.dataset,
        )

        trainer.train()
        trainer.save_model()
